#define IVS_VERSION    "IVS (2.1)  INRIA Videoconferencing System"
#define DEFAULT_TTL            32
#define N_MAX_STATIONS         50
#define LEN_MAX_NAME           60

#define BOOLEAN     unsigned char
#ifndef TRUE
#define TRUE                    1
#endif /* TRUE */
#ifndef FALSE                   
#define FALSE                   0
#endif /* FALSE */

/* between two ivs.c processes */

#define P_DECLARATION           0
#define P_ASK_INFO              1
#define P_ABORT                 2

/* between a video_decoder and a video_coder */

#define P_FULL_INTRA_REQUEST    0x80
#define P_SEND_INFO_PACKET_LOST 0x40


#define PP_QUIT                 0
#define PP_NEW_TABLE            1
#define PP_NEW_FORMAT           2
#define PP_FORK                 3

#define VIDEO_TYPE              0
#define AUDIO_TYPE              2
#define CONTROL_TYPE            3

#define ABORT_AUDIO             1
#define UNMAPP_AUDIO            2
#define MAPP_AUDIO              4
#define ABORT_VIDEO             8
#define UNMAPP_VIDEO           16
#define MAPP_VIDEO             32
#define NEW_STATION            64
#define KILL_STATION          128

#define PCM_64                  0
#define PCM_32                  1
#define ADPCM_32                2
#define VADPCM                  3

#define CIF4_SIZE               0
#define CIF_SIZE                1
#define QCIF_SIZE               2

#define P_MAX               10000

#define PIP_QUIT                0  /* The process must kill himself */
#define PIP_SIZE                1  /* New size (CIF4_SIZE or CIF_SIZE or QCIF_SIZE) */
#define PIP_FREEZE              2  /* Freeze the image */
#define PIP_UNFREEZE            3  /* Unfreeze the image */
#define PIP_WCONTROL            4  /* Without any rate control */
#define PIP_RCONTROL            5  /* Control refresh */
#define PIP_QCONTROL            6  /* Control the quality */
#define PIP_VFC_PORT            7  /* New VFC port */
#define PIP_MAX_RATE            8  /* Maximun bandwidth value */
#define PIP_LDISP               9  /* Local display */
#define PIP_NOLDISP            10  /* No local display */

typedef struct
{
  char name[LEN_MAX_NAME];                /* name@hostname                */
  u_long  sin_addr;                       /* Internet address             */
  BOOLEAN video_encoding, audio_encoding; /* encoder's parameters         */
  BOOLEAN video_decoding, audio_decoding; /* choice of decoding           */
  BOOLEAN valid;
  int     lasttime;                       /* last Packet Declaration time */
} STATION;

typedef STATION LOCAL_TABLE[N_MAX_STATIONS];

typedef struct
{
  u_long  sin_addr;                  /* Internet address                */
  int audio_decoding;                /* TRUE if audio decoding selected */
} AUDIO_STATION;

typedef AUDIO_STATION AUDIO_TABLE[N_MAX_STATIONS];

/* Default port numbers in multicast mode */
static char MULTI_AUDIO_PORT[] =   "2233";
static char MULTI_VIDEO_PORT[] =   "2244";
static char MULTI_CONTROL_PORT[] = "2255";

/* Default multicast address */
static char MULTI_IP_GROUP[] =     "224.8.8.8";

/* Port numbers in unicast mode */
static char UNI_AUDIO_PORT[] =   "2232";
static char UNI_VIDEO_PORT[] =   "2243";
static char UNI_CONTROL_PORT[] = "2254";

/* Port number for video decoders to video coder messages */
#define PORT_VIDEO_CONTROL   2230

static char AUDIO_REPLAY_PORT[] = "2231";
static char VIDEO_REPLAY_PORT[] = "2242";


/* For control rate */

#define WITHOUT_CONTROL         0
#define REFRESH_CONTROL         1
#define QUALITY_CONTROL         2


/* For IVS DAEMON */

#define IVSD_PORT            2241
#define CALL_ACCEPTED           0
#define CALL_REFUSED            1
#define CALL_REQUEST            2
#define CALL_ABORT              3
