/**************************************************************************
*          Copyright (c) 1992 INRIA Sophia Antipolis, FRANCE.              *
*                                                                          *
* Permission to use, copy, modify, and distribute this material for any    *
* purpose and without fee is hereby granted, provided that the above       *
* copyright notice and this permission notice appear in all copies.        *
* WE MAKE NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS     *
* MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS   *
* OR IMPLIED WARRANTIES.                                                   *
\**************************************************************************/
/**************************************************************************\
*                                                                          *
*  File              : videosparc_vfc.c                                    *
*  Last modification : November 1992                                       *
*  Author            : Thierry Turletti                                    *
*--------------------------------------------------------------------------*
*  Description :  Video interface for VIDEOPIX framegrabber.               *
*                                                                          *
* Added support for NTSC, and auto format detection,                       *
* David Berry, Sun Microsystems,                                           *
* david.berry@sun.com                                                      *
*                                                                          *
* The VideoPix specific code is originally from work that is               *
* Copyright (c) 1990-1992 by Sun Microsystems, Inc.                        *
* Permission to use, copy, modify, and distribute this software and        * 
* its documentation for any purpose and without fee is hereby granted,     *
* provided that the above copyright notice appear in all copies            * 
* and that both that copyright notice and this permission notice appear    * 
* in supporting documentation.                                             *
*                                                                          *
* This file is provided AS IS with no warranties of any kind.              * 
* The author shall have no liability with respect to the infringement      *
* of copyrights, trade secrets or any patents by this file or any part     * 
* thereof.  In no event will the author be liable for any lost revenue     *
* or profits or other special, indirect and consequential damages.         *
*--------------------------------------------------------------------------*
* Modified by Winston Dang  wkd@Hawaii.Edu	10/1/92                    *
* alterations were made for speed purposes.                                *
\**************************************************************************/

#ifdef VIDEOPIX

#include "h261.h"
#include "protocol.h"
#include <vfc_lib.h>

#define GET_UBITS        0x00c00000
#define GET_VBITS        0x00300000

/* Transform the Cr,Cb videopix output to CCIR signals.
*  Cr, Cb outputs are twos complement values in [-63,63]
*  CCIR are unsigned values in [16, 240] with 128 <-> 0.
*/
static int COMP2TOCCIR[128]={
  128, 129, 131, 133, 134, 136, 138, 139, 141, 143,
  145, 146, 148, 150, 151, 153, 155, 156, 158, 160,
  162, 163, 165, 167, 168, 170, 172, 173, 175, 177,
  179, 180, 182, 184, 185, 187, 189, 190, 192, 194,
  196, 197, 199, 201, 202, 204, 206, 207, 209, 211,
  213, 214, 216, 218, 219, 221, 223, 224, 226, 228,
  230, 231, 233, 235, 19, 20, 22, 24, 26, 27,
  29, 31, 32, 34, 36, 37, 39, 41, 43, 44,
  46, 48, 49, 51, 53, 54, 56, 58, 60, 61,
  63, 65, 66, 68, 70, 71, 73, 75, 77, 78,
  80, 82, 83, 85, 87, 88, 90, 92, 94, 95,
  97, 99, 100, 102, 104, 105, 107, 109, 111, 112,
  114, 116, 117, 119, 121, 122, 124, 126
  };



extern FILE *F_loss;
extern BOOLEAN STAT_MODE;


Display *dpy;
XImage *ximage;
GC gc;
Window win;
XColor colors[256];
u_char map_lut[256];
int depth;



#define NTSC_FIELD_HEIGHT       240     /* Height of a field in NTSC */
#define PAL_FIELD_HEIGHT        287     /* Height of a field in PAL */
#define HALF_YUV_WIDTH          360     /* Half the width of YUV (720/2) */
#define QCIF_WIDTH              176     /* Width of Quarter CIF */
#define COL_MAX                 352
#define LIG_MAX                 288
#define LIG_MAX_DIV2            144
#define COL_MAX_DIV2            176


void vfc_get_cif_pal(vfc_dev, im_data)
    VfcDev *vfc_dev;
    u_char im_data[][COL_MAX];
  {
    register int i, j;
    register u_char *r_data;
    register u_int tmpdata, *data_port;
    static int WIDTH = QCIF_WIDTH >> 4;
    static int grab_ioctl = CAPTRCMD;
    static int limit = VFC_ESKIP_PAL >> 4;


    /*
    * Local register copy of the data port
    * improves performance
    */
    data_port = vfc_dev->vfc_port2;

    /*
     * Grab the image
     */
    ioctl(vfc_dev->vfc_fd, VFCSCTRL, &grab_ioctl);

    /* Skip over the non-active samples */
    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<limit; i++){
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
    }

    /* Read in the odd field data */
    r_data = (u_char *)im_data;

    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<PAL_FIELD_HEIGHT; i++){
      for (j=0; j< WIDTH; j++){
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;		
      }
      for (j<<=5; j < HALF_YUV_WIDTH; j++) {
	tmpdata = *data_port;
	tmpdata = *data_port;
      }
    }
  }




void vfc_get_cif_color_pal(vfc_dev, Y_data, Cb_data, Cr_data)
    VfcDev *vfc_dev;
    u_char Y_data[][COL_MAX];
    u_char Cb_data[][COL_MAX_DIV2];
    u_char Cr_data[][COL_MAX_DIV2];
  {
    register int i, j;
    register u_char *y_data, *cb_data, *cr_data;
    register u_int tmpdata, *data_port;
    register int pixel0, pixel1, pixel2, pixel3;
    u_char cb, cr;
    static int CIF_WIDTH_DIV2=176; 
    static int grab_ioctl = CAPTRCMD;
    static int limit = VFC_ESKIP_PAL >> 4;

    static int uv65[4] = {0, 32, 64, 96};
    static int uv43[4] = {0, 8, 16, 24};
    static int uv21[4] = {0, 2, 4, 6};
    static int uv0[2] = {0, 1};

    /*
    * Local register copy of the data port
    * improves performance
    */
    data_port = vfc_dev->vfc_port2;

    /*
     * Grab the image
     */
    ioctl(vfc_dev->vfc_fd, VFCSCTRL, &grab_ioctl);

    /* Skip over the non-active samples */
    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<limit; i++){
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
    }

    /* Read in the odd field data */
    y_data =  (u_char *)Y_data;
    cb_data = (u_char *)Cb_data;
    cr_data = (u_char *)Cr_data;

    for(i=0; i<PAL_FIELD_HEIGHT; i++){
      /* Drop the first pixel */
      *y_data++ = VFC_PREVIEW_Y(*data_port, 30);
      tmpdata = *data_port;
      *y_data++ = VFC_PREVIEW_Y(*data_port, 30);
      *cb_data++ = 0;
      *cr_data++ = 0;
      for(j=1; j<CIF_WIDTH_DIV2; j++){
	pixel0 = *data_port;
	pixel1 = *data_port;
	pixel2 = *data_port;
	pixel3 = *data_port;
	cb = COMP2TOCCIR[
	     uv65[(pixel0 & GET_UBITS)>>22] | uv43[(pixel1 & GET_UBITS)>>22] |
	     uv21[(pixel2 & GET_UBITS)>>22] | uv0[(pixel3 & GET_UBITS)>>23]];
	cr = COMP2TOCCIR[
             uv65[(pixel0 & GET_VBITS)>>20] | uv43[(pixel1 & GET_VBITS)>>20] +
	     uv21[(pixel2 & GET_VBITS)>>20] | uv0[(pixel3 & GET_VBITS)>>21]];

	*y_data++ = VFC_PREVIEW_Y(pixel0, 30);
	*y_data++ = VFC_PREVIEW_Y(pixel2, 30);
	*cb_data++ = cb;
	*cr_data++ = cr;
	
      }
      tmpdata = *data_port;
      for(j <<= 1; j < HALF_YUV_WIDTH; j++){
	tmpdata = *data_port;
	tmpdata = *data_port;
      }
    }
  }
   


void vfc_get_cif4_pal(vfc_dev, im_data)
    VfcDev *vfc_dev;
    u_char im_data[][LIG_MAX][COL_MAX];
  {
    register int i, j;
    register u_char *r_data0, *r_data1, *r_data2, *r_data3;
    register u_int tmpdata, *data_port1, *data_port2;
    static int WIDTH = QCIF_WIDTH >> 4;
    static int PAL_FIELD_HEIGHT_DIV2 = PAL_FIELD_HEIGHT >> 1;
    static int grab_ioctl = CAPTRCMD;
    static int limit = VFC_ESKIP_PAL >> 4;


    /*
     * Grab the image
     */
    ioctl(vfc_dev->vfc_fd, VFCSCTRL, &grab_ioctl);

    data_port1 = vfc_dev->vfc_port1;
    data_port2 = vfc_dev->vfc_port2;

    /* Skip over the non-active samples */
    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<limit; i++){
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;	
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;	
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;	

      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;	
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;	
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;	
    }

    r_data0 = (u_char *)im_data[0]; /* CIF0 image */
    r_data1 = (u_char *)im_data[1]; /* CIF1 image */
    r_data2 = (u_char *)im_data[2]; /* CIF2 image */
    r_data3 = (u_char *)im_data[3]; /* CIF3 image */

    /* looks gross but there is less time spent loop than reading */

    /* Read in the even/odd fields data in CIF0 & CIF1*/

    for(i=0; i<PAL_FIELD_HEIGHT_DIV2; i++){

      for (j=0; j<WIDTH; j++){
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for (j=0; j<WIDTH; j++){
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
      }
      j <<= 5;
      for(; j<HALF_YUV_WIDTH; j++){
	tmpdata = *data_port1;
	tmpdata = *data_port1;
	tmpdata = *data_port2;
	tmpdata = *data_port2;
      }
    }

    /* Read in the even/odd fields data in CIF2 & CIF3 */

    for(i=0; i<PAL_FIELD_HEIGHT_DIV2; i++){

      for (j=0; j<WIDTH; j++){
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for (j=0; j<WIDTH; j++){
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);

      }
      j <<= 5;
      for(; j<HALF_YUV_WIDTH; j++){
	tmpdata = *data_port1;
	tmpdata = *data_port1;
	tmpdata = *data_port2;
	tmpdata = *data_port2;
      }
    }
  }



void vfc_get_cif4_ntsc(vfc_dev, im_data)
    VfcDev *vfc_dev;
    u_char im_data[][LIG_MAX][COL_MAX];
  {
    register int i, j;
    register u_char *r_data0, *r_data1, *r_data2, *r_data3;
    register u_int tmpdata, *data_port1, *data_port2;
    static int WIDTH = QCIF_WIDTH >> 4;
    static int NTSC_FIELD_HEIGHT_DIV2 = NTSC_FIELD_HEIGHT >> 1;
    static int grab_ioctl = CAPTRCMD;
    static int limit = VFC_OSKIP_NTSC >> 4;


    /*
     * Grab the image
     */
    ioctl(vfc_dev->vfc_fd, VFCSCTRL, &grab_ioctl);

    data_port1 = vfc_dev->vfc_port1;
    data_port2 = vfc_dev->vfc_port2;

    /* Skip over the non-active samples */
    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<limit; i++){
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;	
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;	
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;
      tmpdata = *data_port1;	

      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;	
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;	
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;
      tmpdata = *data_port2;	
    }

    r_data0 = (u_char *)im_data[0]; /* CIF0 image */
    r_data1 = (u_char *)im_data[1]; /* CIF1 image */
    r_data2 = (u_char *)im_data[2]; /* CIF2 image */
    r_data3 = (u_char *)im_data[3]; /* CIF3 image */

    /* looks gross but there is less time spent loop than reading */

    /* Read in the even/odd fields data in CIF0 & CIF1*/

    for(i=0; i<NTSC_FIELD_HEIGHT_DIV2; i++){
      for (j=0; j<WIDTH; j++){
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for (j=0; j<WIDTH; j++){
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data0++ = VFC_PREVIEW_Y(*data_port1, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data1++ = VFC_PREVIEW_Y(*data_port1, 30);
      }
      j <<= 5;
      for(; j<HALF_YUV_WIDTH; j++){
	tmpdata = *data_port1;
	tmpdata = *data_port1;
	tmpdata = *data_port2;
	tmpdata = *data_port2;
      }
    }

    /* Read in the even/odd fields data in CIF2 & CIF3 */

    for(i=0; i<NTSC_FIELD_HEIGHT_DIV2; i++){

      for (j=0; j<WIDTH; j++){
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for (j=0; j<WIDTH; j++){
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port2, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data2++ = VFC_PREVIEW_Y(*data_port1, 30);
      }
      for(j=0; j<WIDTH; j++){
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
	*r_data3++ = VFC_PREVIEW_Y(*data_port1, 30);
      }
      j <<= 5;
      for(; j<HALF_YUV_WIDTH; j++){
	tmpdata = *data_port1;
	tmpdata = *data_port1;
	tmpdata = *data_port2;
	tmpdata = *data_port2;
      }
    }
  }



void vfc_get_cif_ntsc(vfc_dev, im_data)
    VfcDev *vfc_dev;
    u_char im_data[][COL_MAX];
  {
    register int i, j;
    register u_char *r_data;
    register u_int tmpdata, *data_port;
    static int grab_ioctl = CAPTRCMD;
    static int WIDTH = QCIF_WIDTH >> 4;
    static int limit = VFC_OSKIP_NTSC >> 4;

    /*
    * Local register copy of the data port
    * improves performance
    */
    data_port = vfc_dev->vfc_port1;

    /*
     * Grab the image
     */
    ioctl(vfc_dev->vfc_fd, VFCSCTRL, &grab_ioctl);

    /* Skip over the non-active samples */
    for (i=0; i<limit; i++){
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
    }

    
    /* Read in the odd field data */
    r_data = (u_char *)im_data;

    for(i=0; i<NTSC_FIELD_HEIGHT; i++){
      for(j=0; j< WIDTH; j++){
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;		
      }
      j <<= 5;
      while (j++ < HALF_YUV_WIDTH) {
	tmpdata = *data_port;
	tmpdata = *data_port;	
    }
  }
}




void vfc_get_qcif_pal(vfc_dev, im_data)
    VfcDev *vfc_dev;
    u_char im_data[][COL_MAX];
  {
    register int i, j;
    register u_char *r_data;
    register u_int tmpdata, *data_port;
    static int WIDTH = QCIF_WIDTH >> 4;
    static int grab_ioctl = CAPTRCMD;
    static int limit = VFC_ESKIP_PAL >> 4;
    static int PAL_FIELD_HEIGHT_DIV2 = PAL_FIELD_HEIGHT >> 1;
    static int VFC_YUV_WIDTH_DIV64 = VFC_YUV_WIDTH >> 6;

    /*
    * Local register copy of the data port
    * improves performance
    */
    data_port = vfc_dev->vfc_port2;

    /*
     * Grab the image
     */
    ioctl(vfc_dev->vfc_fd, VFCSCTRL, &grab_ioctl);

    /* Skip over the non-active samples */
    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<limit; i++){
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
    }

    /* Read in the odd field data */
    r_data = (u_char *)im_data;

    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<PAL_FIELD_HEIGHT_DIV2; i++){
      for(j=0; j<WIDTH; j++){
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
      }
      r_data += QCIF_WIDTH;

      /* Drop the next line + 16 pixels */
      /* One line = VFC_YUV_WIDTH = HALF_YUV_WIDTH+16 */
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
      for(j=0; j<VFC_YUV_WIDTH_DIV64; j++){
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
      }
    }
  }






void vfc_get_qcif_ntsc(vfc_dev, im_data)
    VfcDev *vfc_dev;
    u_char im_data[][COL_MAX];
  {
    register int i, j;
    register u_char *r_data;
    register u_int tmpdata, *data_port;
    static int WIDTH = QCIF_WIDTH >> 4;
    static int grab_ioctl = CAPTRCMD;
    static int limit = VFC_OSKIP_NTSC >> 4;
    static int NTSC_FIELD_HEIGHT_DIV2 = NTSC_FIELD_HEIGHT >> 1;
    static int VFC_YUV_WIDTH_DIV64 = VFC_YUV_WIDTH >> 6;

    /*
    * Local register copy of the data port
    * improves performance
    */
    data_port = vfc_dev->vfc_port2;

    /*
     * Grab the image
     */
    ioctl(vfc_dev->vfc_fd, VFCSCTRL, &grab_ioctl);

    /* Skip over the non-active samples */
    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<limit; i++){
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;
      tmpdata = *data_port;	
    }

    /* Read in the odd field data */
    r_data = (u_char *)im_data;

    /* looks gross but there is less time spent loop than reading */
    for(i=0; i<NTSC_FIELD_HEIGHT_DIV2; i++){
      for(j=0; j<WIDTH; j++){
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	*r_data++ = VFC_PREVIEW_Y(*data_port, 30);
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
      }
      r_data += QCIF_WIDTH;

      /* Drop the next line + 16 pixels */
      /* One line = VFC_YUV_WIDTH = HALF_YUV_WIDTH+16 */
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
      for(j=0; j<VFC_YUV_WIDTH_DIV64; j++){
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
	tmpdata = *data_port;
      }
    }
  }



int VfcScanPorts(vfc_dev, format)
     VfcDev  *vfc_dev;
     int     *format;
{
  int in_port;

  for(in_port=VFC_PORT1; in_port<VFC_SVIDEO; in_port++){
    vfc_set_port(vfc_dev, in_port);
    if(vfc_set_format(vfc_dev, VFC_AUTO, format) >= 0)
      return(in_port);
  }
  /* in_port == VFC_SVIDEO */
  vfc_set_port(vfc_dev, in_port);
  if(vfc_set_format(vfc_dev, VFC_AUTO, format) >= 0){
    *format = VFC_NTSC;
    return(in_port);
  }else{
    /* No video available */
    return(-1);
  }
}



void Initialize(vfc_format, size, COLOR, NG_MAX, L_lig, L_col, Lcoldiv2,
		L_col2, L_lig2, get_image)
     int vfc_format, size;
     BOOLEAN COLOR;
     int *NG_MAX, *L_lig, *L_col, *Lcoldiv2, *L_col2, *L_lig2;
     void (**get_image)();
{
  if(vfc_format == VFC_PAL){
    if(size == QCIF_SIZE){
      *NG_MAX = 5;
      *L_lig = 144;
      *L_col = 176;
    }else{
      *NG_MAX = 12;
      *L_lig = 288;
      *L_col = 352;
    }
  }else{
    if(size == QCIF_SIZE){
      *NG_MAX = 5;
      *L_lig = 112;
      *L_col = 176;
    }else{
      *NG_MAX = 10;
      *L_lig = 240;
      *L_col = 352;
    }
  }
  *Lcoldiv2 = *L_col >> 1;
  *L_col2 = *L_col << 1;
  *L_lig2 = *L_lig << 1;

  if(!COLOR){
    switch(size){
    case CIF4_SIZE: *get_image = (vfc_format == VFC_NTSC) ? 
      vfc_get_cif4_ntsc : vfc_get_cif4_pal;
      break;
    case CIF_SIZE:
      *get_image = (vfc_format == VFC_NTSC) ? vfc_get_cif_ntsc :
	vfc_get_cif_pal;
      break;
    case QCIF_SIZE:
      *get_image = (vfc_format == VFC_NTSC) ? vfc_get_qcif_ntsc :
	vfc_get_qcif_pal;
    }
  }else{
    switch(size){
    case CIF4_SIZE: 
      *get_image = (vfc_format == VFC_NTSC) ? vfc_get_cif4_ntsc : 
	vfc_get_cif4_pal;
      break;
    case CIF_SIZE:
      *get_image = (vfc_format == VFC_NTSC) ? vfc_get_cif_ntsc :
	vfc_get_cif_color_pal;
      break;
    case QCIF_SIZE:
      *get_image = (vfc_format == VFC_NTSC) ? vfc_get_qcif_ntsc :
	vfc_get_qcif_pal;
    }
  }
}


void video_encode(group, port, ttl, size, fd_f2s, fd_s2f, COLOR, STAT, 
		  FREEZE, LOCAL_DISPLAY, display, rate_control, 
		  rate_max, vfc_port, vfc_format, vfc_dev)
     char *group, *port;      /* Address socket */
     u_char ttl;              /* ttl value */
     int size;                /* image's size */
     int fd_f2s[2];           /* pipe descriptor father to son */
     int fd_s2f[2];           /* pipe descriptor son to father*/
     BOOLEAN COLOR;           /* True if color encoding */
     BOOLEAN STAT;            /* True if statistical mode */
     BOOLEAN FREEZE;          /* True if image frozen */
     BOOLEAN LOCAL_DISPLAY;   /* True if local display selected */
     char *display;           /* Display name */
     int rate_control;        /* WITHOUT_CONTROL or REFRESH_CONTROL 
				 or QUALITY_CONTROL */
     int rate_max;            /* Maximum bandwidth allowed (kb/s) */
     int vfc_port;            /* VFC input port selected */
     int vfc_format;          /* VFC_PAL or VFC_NTSC */
     VfcDev *vfc_dev;         /* VFC device (already opened) */

{
    extern int n_mba, n_tcoeff, n_cbc, n_dvm;
    extern int tab_rec1[][255], tab_rec2[][255]; /* for quantization */
    IMAGE_CIF image_coeff[4];
    u_char new_Y[4][LIG_MAX][COL_MAX];
    u_char new_Cb[4][LIG_MAX_DIV2][COL_MAX_DIV2];
    u_char new_Cr[4][LIG_MAX_DIV2][COL_MAX_DIV2];
    int NG_MAX;
    int sock_recv=0;
    BOOLEAN FIRST=TRUE; /* True if it is the first image */
    BOOLEAN CONTINUE=TRUE;
    BOOLEAN NEW_LOCAL_DISPLAY, NEW_SIZE;
    int i;
    int new_size, new_vfc_port;
    void (*get_image)();
    int L_lig, L_col, L_lig2, L_col2, Lcoldiv2;

    /* Video control socket declaration */
    sock_recv=declare_listen(PORT_VIDEO_CONTROL);
    if(STAT){
      if((F_loss=fopen(".videoconf.loss", "w")) == NULL){
	fprintf(stderr, "cannot create videoconf.loss file\n");
	STAT_MODE=FALSE;
      }else
	STAT_MODE=TRUE;
    }

    Initialize(vfc_format, size, COLOR, &NG_MAX, &L_lig, &L_col, &Lcoldiv2,
	       &L_col2, &L_lig2, &get_image);

    if(LOCAL_DISPLAY){
      if(size == CIF4_SIZE){
	init_window(L_lig2, L_col2, "Local display", map_lut, display, 
		    FALSE, &depth, &dpy, &ximage, colors);
      }else{
	init_window(L_lig, L_col, "Local display", map_lut, display, COLOR,
		    &depth, &dpy, &ximage, colors);
      }
    }

    do{
      if(!COLOR){
	if(size != CIF4_SIZE){
	  if(!FREEZE)
	    (*get_image)(vfc_dev, new_Y[0]);
	  encode_h261(group, port, ttl, new_Y[0], new_Cb[0], new_Cr[0],
		      image_coeff[0], NG_MAX, sock_recv, FIRST, COLOR, 
		      vfc_format, rate_control, rate_max, fd_s2f[1]);
	  if(LOCAL_DISPLAY){
	    register int l, c;
	    register u_char *im_pix = (u_char *) ximage->data;
	    if(depth != 4){
	      register u_char *im_Y;
	      
	      for(l=0; l<L_lig; l++){
		im_Y = &new_Y[0][l][0];
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y++];
	      }
	    }else{
	      register u_char *im_Y;
	      
	      for(l=0; l<L_lig; l++){
		im_Y = &new_Y[0][l][0];
		for(c=0; c<Lcoldiv2; c++)
		  *im_pix++ = map_lut[((*im_Y++ >> 4) << 4) |
				      (*im_Y++ >> 4)];
	      }
	    }
	    show(dpy, ximage, L_col, L_lig);
	  }
	}else{
	  /* Super CIF image encoding */
	  if(!FREEZE)
	    (*get_image)(vfc_dev, new_Y);
	  for(i=0; i<4; i++){
	    encode_h261_cif4(group, port, ttl, new_Y[i], new_Cb[i], 
			     new_Cr[i], (GOB *)image_coeff[i], sock_recv, 
			     FIRST, COLOR, i, vfc_format, rate_control, 
			     rate_max, fd_s2f[1]);
	  }
	  if(LOCAL_DISPLAY){
	    int lmax=L_lig-3;
	    register int l, c ;
	    register u_char *im_pix = (u_char *) ximage->data;
	    if(depth != 4){
	      register u_char *im_Y0 = &new_Y[0][0][0];
	      register u_char *im_Y1 = &new_Y[1][0][0];

	      for(l=0; l<lmax; l++){
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y0++];
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y1++];
	      }
	      im_Y0 = (u_char *) &new_Y[2][0][0];
	      im_Y1 = (u_char *) &new_Y[3][0][0];
	      for(l=0; l<lmax; l++){
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y0++];
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y1++];
	      }
	    }else{
	      /* depth == 4 */
	      register u_char *im_Y0 = &new_Y[0][0][0];
	      register u_char *im_Y1 = &new_Y[1][0][0];
		
	      for(l=0; l<lmax; l++){
		for(c=0; c<Lcoldiv2; c++)
		  *im_pix++ = map_lut[((*im_Y0++ >> 4) << 4) |
				      (*im_Y0++ >> 4)];
		for(c=0; c<Lcoldiv2; c++)
		  *im_pix++ = map_lut[((*im_Y1++ >> 4) << 4) |
				      (*im_Y1++ >> 4)];
	      }
	      im_Y0 = (u_char *) &new_Y[2][0][0];
	      im_Y1 = (u_char *) &new_Y[3][0][0];
	      for(l=0; l<lmax; l++){
		for(c=0; c<Lcoldiv2; c++)
		  *im_pix++ = map_lut[((*im_Y0++ >> 4) << 4) |
				      (*im_Y0++ >> 4)];
		for(c=0; c<Lcoldiv2; c++)
		  *im_pix++ = map_lut[((*im_Y1++ >> 4) << 4) |
				      (*im_Y1++ >> 4)];
	      }
	    }
	    show(dpy, ximage, L_col2, L_lig2);
	  } /* if(local display) */
	} /* else{} Super CIF encoding */
      }else{
	/* Color encoding */
	if(size != CIF4_SIZE){
	  if(!FREEZE)
	    (*get_image)(vfc_dev, new_Y[0], new_Cb[0], new_Cr[0]);
	  if(LOCAL_DISPLAY){
	    register int l, c;
	    register u_char *im_pix = (u_char *) ximage->data;
	    if(depth == 8){
	      register u_char *im_Y = &new_Y[0][0][0];
	      register u_char *im_Cb = &new_Cb[0][0][0];
	      register u_char *im_Cr = &new_Cr[0][0][0];
	      int val_Cb, val_Cr;

	      for(l=0; l<L_lig; l++){
		for(c=0; c<L_col; c+=2){
		  val_Cb = ((*im_Cb++ & 0xe0) >> 2);
		  val_Cr = ((*im_Cr++ & 0xe0) >> 5);
		  *im_pix++ = map_lut[yuv_to_lut[((*im_Y++ & 0xf0)<<2) |
						 val_Cb | val_Cr]];
		  *im_pix++ = map_lut[yuv_to_lut[((*im_Y++ & 0xf0)<<2) |
						 val_Cb | val_Cr]];
		}
	      }
	    }else{
	      /* depth == 4 */
	    }
	    show(dpy, ximage, L_col, L_lig);
	  } /* if(LOCAL_DISPLAY) */
	  encode_h261(group, port, ttl, new_Y[0], new_Cb[0], new_Cr[0],
		      image_coeff[0], NG_MAX, sock_recv, FIRST, COLOR, 
		      vfc_format, rate_control, rate_max, fd_s2f[1]);
	}else{
	  /* Super CIF image encoding */
	  if(!FREEZE)
	    (*get_image)(vfc_dev, new_Y);
	  if(LOCAL_DISPLAY){
	    int lmax=L_lig-3;
	    register int l, c ;
	    register u_char *im_pix = (u_char *) ximage->data;
	    if(depth == 8){
	      register u_char *im_Y0;
	      register u_char *im_Y1;

	      im_Y0 = (u_char *) &new_Y[0][0][0];
	      im_Y1 = (u_char *) &new_Y[1][0][0];
	      for(l=0; l<lmax; l++){
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y0++];
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y1++];
	      }
	      im_Y0 = (u_char *) &new_Y[2][0][0];
	      im_Y1 = (u_char *) &new_Y[3][0][0];
	      for(l=0; l<lmax; l++){
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y0++];
		for(c=0; c<L_col; c++)
		  *im_pix++ = map_lut[*im_Y1++];
	      }
	    }else{
	      /* depth == 4 */
	    }
	    show(dpy, ximage, L_col2, L_lig2);
	  } /* if(LOCAL_DISPLAY) */
	  for(i=0; i<4; i++){
	    encode_h261_cif4(group, port, ttl, new_Y[i], new_Cb[i], new_Cr[i],
			     image_coeff[i], sock_recv, FIRST, COLOR, i, 
			     vfc_format, rate_control, rate_max, fd_s2f[1]);
	  }
	} /* else{} Super CIF encoding */
      } /* else{} Color encoding */
      FIRST = FALSE;
      new_size = size;
      new_vfc_port = vfc_port;
      NEW_LOCAL_DISPLAY = LOCAL_DISPLAY;
      NEW_SIZE = FALSE;
      if(CONTINUE = HandleCommands(fd_f2s, &new_size, &rate_control, 
				&new_vfc_port, &rate_max, &FREEZE, 
				&NEW_LOCAL_DISPLAY)){
	if(new_vfc_port != vfc_port){
	  vfc_port = new_vfc_port;
	  vfc_set_port(vfc_dev, vfc_port);
	}
	if(new_size != size){
	  NEW_SIZE = TRUE;
	  FIRST = TRUE;
	  size = new_size;
	  Initialize(vfc_format, size, COLOR, &NG_MAX, &L_lig, &L_col, 
		     &Lcoldiv2, &L_col2, &L_lig2, &get_image);
	  if(LOCAL_DISPLAY)
	    kill_window(dpy, ximage);
	}
	if((NEW_LOCAL_DISPLAY != LOCAL_DISPLAY) || NEW_SIZE){
	  LOCAL_DISPLAY = NEW_LOCAL_DISPLAY;
	  if(LOCAL_DISPLAY){
	    if(size == CIF4_SIZE){
	      init_window(L_lig2, L_col2, "Local display", map_lut, display, 
			  FALSE, &depth, &dpy, &ximage, colors);
	    }else{
	      init_window(L_lig, L_col, "Local display", map_lut, display, 
			  COLOR, &depth, &dpy, &ximage, colors);
	    }
	  }
	  else
	    kill_window(dpy, ximage); 
	}
      }
    }while(CONTINUE);

    /* Clean up and exit */
    close(fd_f2s[0]);
    close(fd_s2f[1]);
    if(LOCAL_DISPLAY)
      XCloseDisplay(dpy);
    if(STAT_MODE)
      fclose(F_loss);
    vfc_destroy(vfc_dev);
    exit(0);
    /* NOT REACHED */
}

#endif /* VIDEOPIX */
